<?php

namespace CRM\Factory;

use Doctrine\ORM\EntityManager;
use CRM\Traits\TQuery;

class CrmDashboardFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function dataBaseChartPieFactory($cdoperador, $nrorg)
    {
        $sql = "SELECT
        FASE.NMSEQFASENEGO AS FASE,
        COUNT(OPOR.NRSEQOPORT) AS QUANTIDADE
    FROM
        CRM_OPORTUNI OPOR
    JOIN CRM_FASENEGOCI FASE ON
        OPOR.NRORG = FASE.NRORG
        AND OPOR.NRSEQFASENEGO = FASE.NRSEQFASENEGO
    WHERE
        OPOR.DTREALFECHA IS NULL
        AND OPOR.NRORG = ({$nrorg})
        AND OPOR.NRPARCRESP IN({$cdoperador})
    GROUP BY
        FASE.NMSEQFASENEGO
    ORDER BY
 	    COUNT(OPOR.NRSEQOPORT) ASC
    ";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function dataBaseChartBarLeadFactory($nrorg)
    {
        $sql = "SELECT
        COUNT(CANDIDATO.NRCANDIDATO) LEADSCADASTRADOS,
        COUNT(PROSPECT.CDPROSPECT) LEADSCONVERTIDOS,
        TO_CHAR(TRUNC(CANDIDATO.DTINCLUSAO, 'MONTH'), 'Month YYYY') MES,
        TO_CHAR(CANDIDATO.DTINCLUSAO, 'MM/YY') ORDEM
    FROM
        CRM_CANDIDATO CANDIDATO
    LEFT JOIN PROSPECT ON
        CANDIDATO.NRPARCNEGOCIO = PROSPECT.NRPARCNEGOCIO
        AND PROSPECT.NRORG = CANDIDATO.NRORG
    WHERE
        CANDIDATO.DTINCLUSAO > ADD_MONTHS(TRUNC(SYSDATE, 'MONTH'), -6)
        AND CANDIDATO.NRORG = ({$nrorg})
    GROUP BY
        TRUNC(CANDIDATO.DTINCLUSAO, 'MONTH'),
        TO_CHAR(CANDIDATO.DTINCLUSAO, 'MM/YY')";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function dataBaseChartThermometerFactory($cdoperador, $nrorg)
    {
        $sql = "SELECT
        SUM(QUANTIDADE) AS QUANTIDADE,
        DECODE(TERMOMETRO, NULL, 'Sem Termômetro', TERMOMETRO) TERMOMETRO,
        DECODE(CORTERMOMETRO, NULL, '#ddd', CORTERMOMETRO) CORTERMOMETRO
    FROM
        (
        SELECT
            0 AS QUANTIDADE,
            NMTERMOPORTUNI AS TERMOMETRO,
            DSCORHEXADECIMAL AS CORTERMOMETRO
        FROM
            CRM_TERMOPORTUNI
        WHERE
            NRORG = ({$nrorg})
    UNION ALL
        SELECT
            COUNT(NRSEQOPORT) AS QUANTIDADE,
            NMTERMOPORTUNI AS TERMOMETRO,
            DSCORHEXADECIMAL AS CORTERMOMETRO
        FROM
            CRM_OPORTUNI OPORT
        LEFT JOIN CRM_TERMOPORTUNI TERM ON
            TERM.NRTERMOPORTUNI = OPORT.NRTERMOPORTUNI
            AND TERM.NRORG = OPORT.NRORG
        LEFT JOIN PARCNEGOCIO PARC ON
            PARC.NRPARCNEGOCIO = OPORT.NRPARCRESP
            AND PARC.NRORG = OPORT.NRORG
        WHERE
		OPORT.NRORG = ({$nrorg})
		AND OPORT.CRM_NRORGTRAB = ({$nrorg})
		AND OPORT.DTREALFECHA IS NULL
        GROUP BY
            OPORT.NRTERMOPORTUNI,
            NMTERMOPORTUNI,
            DSCORHEXADECIMAL)
    GROUP BY
        TERMOMETRO,
        CORTERMOMETRO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function dataBaseCardTaskFactory($cdoperadorAsString)
    {
        try {
            $sql = "SELECT
        status_principal AS STATUS,
        SUM(quantidade) AS QUANTIDADE
    FROM
        (
        SELECT
            status_principal AS STATUS_PRINCIPAL,
            COUNT(status_principal) AS QUANTIDADE
        FROM
            (
            SELECT
                CASE
                    WHEN (TO_CHAR(dtplanfilaconta, 'YYYY') || TO_CHAR(dtplanfilaconta, 'MM') || TO_CHAR(dtplanfilaconta, 'DD') < TO_CHAR(CURRENT_DATE, 'YYYY') || TO_CHAR(CURRENT_DATE, 'MM') || TO_CHAR(CURRENT_DATE, 'DD'))
                    AND DTREALFILACONTA IS NULL THEN 'Atrasadas'
                    WHEN ( TO_CHAR(dtplanfilaconta, 'YYYY') || TO_CHAR(dtplanfilaconta, 'MM') || TO_CHAR(dtplanfilaconta, 'DD') > TO_CHAR(CURRENT_DATE, 'YYYY') || TO_CHAR(CURRENT_DATE, 'MM') || TO_CHAR(CURRENT_DATE, 'DD')) THEN 'Futuras'
                    WHEN TO_CHAR(dtplanfilaconta, 'YYYY') || TO_CHAR(dtplanfilaconta, 'MM') || TO_CHAR(dtplanfilaconta, 'DD') = TO_CHAR(CURRENT_DATE, 'YYYY') || TO_CHAR(CURRENT_DATE, 'MM') || TO_CHAR(CURRENT_DATE, 'DD') THEN 'Hoje'
                END STATUS_PRINCIPAL,
                CDOPERREAL AS OPERADOR
            FROM
                FILACONTATO
            LEFT JOIN CONTATO ON
                FILACONTATO.NRSEQCONTATO = CONTATO.NRSEQCONTATO
                AND CONTATO.NRORG = FILACONTATO.NRORG
            WHERE
                CDOPERREAL = ({$cdoperadorAsString}) 
                AND DTREALFILACONTA IS NULL
                AND FILACONTATO.IDATIVO = 'S'
                AND (CONTATO.IDTIPOCONTATO != ('OPORTUNIDADE')
                    OR CONTATO.IDTIPOCONTATO IS NULL))
        GROUP BY
            status_principal
    UNION ALL (
        SELECT
            'Atrasadas' AS STATUS_PRINCIPAL,
            0 AS QUANTIDADE
        FROM
            dual
    UNION ALL
        SELECT
            'Hoje' AS STATUS_PRINCIPAL,
            0 AS QUANTIDADE
        FROM
            dual
    UNION ALL
        SELECT
            'Futuras' AS STATUS_PRINCIPAL,
            0 AS QUANTIDADE
        FROM
            dual ) )
    WHERE
        STATUS_PRINCIPAL IS NOT NULL
    GROUP BY
        status_principal
    ORDER BY
        status_principal ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function dataBaseCardCommitmentFactory($cdoperadorAsString)
    {
        try {
            $sql = "SELECT
        status_principal AS STATUS,
        Sum(quantidade) AS QUANTIDADE
    FROM
        (
        SELECT
            status_principal AS STATUS_PRINCIPAL,
            Count(status_principal) AS QUANTIDADE
        FROM
            (
            SELECT
                CASE
                    WHEN trunc(dtiniplanocupa) = trunc(sysdate) THEN 'Hoje'
                    WHEN trunc(dtiniplanocupa) > trunc(sysdate) THEN 'Futuros'
                END STATUS_PRINCIPAL,
                planocupacao.cdoperador AS OPERADOR
            FROM
                planocupacao
            LEFT JOIN partplanocupa ON
                partplanocupa.nrseqplanocupa = planocupacao.nrseqplanocupa
            WHERE
                ( planocupacao.cdoperador = ({$cdoperadorAsString})
                    OR partplanocupa.cdoperador = ({$cdoperadorAsString}) )
                AND planocupacao.idativo = 'S')
        GROUP BY
            status_principal
    UNION ALL (
        SELECT
            'Hoje' AS STATUS_PRINCIPAL,
            0 AS QUANTIDADE
        FROM
            dual
    UNION ALL
        SELECT
            'Futuros' AS STATUS_PRINCIPAL,
            0 AS QUANTIDADE
        FROM
            dual
    UNION ALL
        SELECT
            'Pendentes' AS STATUS_PRINCIPAL,
            0 AS QUANTIDADE
        FROM
            dual
    UNION ALL
        SELECT
            status_principal AS STATUS_PRINCIPAL,
            Count(status_principal) AS QUANTIDADE
        FROM
            (
            SELECT
                CASE
                    WHEN idstatplanocupa = 'P' THEN 'Pendentes'
                    ELSE NULL
                END STATUS_PRINCIPAL,
                planocupacao.cdoperador AS OPERADOR
            FROM
                planocupacao
            LEFT JOIN partplanocupa ON
                partplanocupa.nrseqplanocupa = planocupacao.nrseqplanocupa
            WHERE
                ( planocupacao.cdoperador = ({$cdoperadorAsString})
                    OR partplanocupa.cdoperador = ({$cdoperadorAsString}) ))
        GROUP BY
            status_principal))
    WHERE
        status_principal IS NOT NULL
    GROUP BY
        status_principal
    ORDER BY
        status_principal DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function dataBaseCardOpportunityFactory($cdoperadorAsString)
    {
        $sql = "SELECT
        SUM(QUANTIDADE) QUANTIDADE,
        STATUS,
        NRORDER
    FROM
        (
        SELECT
            COUNT(NRSEQOPORT) AS QUANTIDADE,
            STATUS,
            NRORDER
        FROM
            (
            SELECT
                OPORT.NRSEQOPORT,
                'Em Aberto' AS STATUS,
                1 AS NRORDER
            FROM
                crm_oportuni OPORT
            JOIN operador OP ON
                OP.nrparcnegocio = OPORT.nrparcresp
                AND OP.nrorg = OPORT.nrorg
            WHERE
                OP.cdoperador = ({$cdoperadorAsString})
                AND OPORT.DTREALFECHA IS NULL
                AND OPORT.IDATIVO = 'S')
    UNION ALL
        SELECT
            COUNT(NRSEQOPORT) AS QUANTIDADE,
            STATUS,
            NRORDER
        FROM
            (
            SELECT
                OPORT.NRSEQOPORT,
                'Abertas no Mês' AS STATUS,
                2 AS NRORDER
            FROM
                crm_oportuni OPORT
            JOIN operador OP ON
                OP.nrparcnegocio = OPORT.nrparcresp
                AND OP.nrorg = OPORT.nrorg
            WHERE
                OP.cdoperador = ({$cdoperadorAsString})
                AND OPORT.DTREALFECHA IS NULL
                AND OPORT.IDATIVO = 'S'
                AND TO_CHAR(OPORT.DTINCLUSAO, 'MM') = TO_CHAR(SYSDATE, 'MM') )
    UNION ALL
        SELECT
            COUNT(NRSEQOPORT) AS QUANTIDADE,
            STATUS,
            NRORDER
        FROM
            (
            SELECT
                OPORT.NRSEQOPORT,
                'Fechadas no Mês' AS STATUS,
                3 AS NRORDER
            FROM
                crm_oportuni OPORT
            JOIN operador OP ON
                OP.nrparcnegocio = OPORT.nrparcresp
                AND OP.nrorg = OPORT.nrorg
            WHERE
                OP.cdoperador = ({$cdoperadorAsString})
                AND OPORT.DTREALFECHA IS NOT NULL
                AND OPORT.IDATIVO = 'S'
                AND TO_CHAR(OPORT.DTINCLUSAO, 'MM') = TO_CHAR(SYSDATE, 'MM') )
    UNION ALL (
        SELECT
            0 AS QUANTIDADE,
            'Em Aberto' AS STATUS,
            1 AS NRORDER
        FROM
            DUAL
    UNION ALL
        SELECT
            0 AS QUANTIDADE,
            'Abertas no Mês' AS STATUS,
            2 AS NRORDER
        FROM
            DUAL
    UNION ALL
        SELECT
            0 AS QUANTIDADE,
            'Fechadas no Mês' AS STATUS,
            3 AS NRORDER
        FROM
            DUAL ) )
    GROUP BY
        STATUS,
        NRORDER
    ORDER BY
        NRORDER";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function dataBaseCardOccurrencesFactory($cdoperadorAsString, $nrorgAsString)
    {
        $sql = "SELECT
        SUM(QUANTIDADE) QUANTIDADE,
        STATUS,
        NRORDER
    FROM
        (
        SELECT
            COUNT(NRSEQOPORT) AS QUANTIDADE,
            STATUS,
            NRORDER
        FROM
            (
            SELECT
                OPORT.NRSEQOPORT,
                EVOL.CDOPERADORGEST,
                'Pendentes' AS STATUS,
                1 AS NRORDER
            FROM
                CRM_OPORTUNI OPORT
            JOIN EVOLCONT EVOL ON
                EVOL.NRCONTATO = OPORT.NRCONTATO
                AND EVOL.NRORG = OPORT.NRORG
            WHERE
                OPORT.DTREALFECHA IS NULL
                AND EVOL.NRSEQFASENEGO = ('8')
                AND EVOL.CDOPERADORGEST = ({$cdoperadorAsString})
                AND OPORT.IDATIVO = ('S')
                AND OPORT.CRM_NRORGTRAB = ({$nrorgAsString})
                AND OPORT.CRM_NRORGTRAB = ({$nrorgAsString})))
    GROUP BY
        STATUS,
        NRORDER
    ORDER BY
        NRORDER";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
