import { Utilities } from '@/util/Utilities';
import { Button, DialogService, LoadingService } from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class CrmTermometroDeOportunidadeController {
  public addNewRow({ component }: IEventParam<TekGrid>) {
    const newId = 0;
    component.addNewRow({ NRTERMOPORTUNI: newId, IDATIVO: 'S' }, 'start');
  }

  public canEditRow({ row }: any) {
    return row['.__added_row'] === true;
  }

  public openFilter({ component }: any) {
    const filterButton = Metadata.getInstance<Button>(
      `${component.name}_filterButton`,
    );
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public saveThermometerOpportunity({ component, event }: IEventParam<TekGrid>) {
    event!.preventDefault();
    if (Utilities.validateGrid(component)) {
      LoadingService.show();
      Http.post('/saveThermometerOpportunity', {
        DRG: component.getEditedRows(),
      }).then(() => {
        component.datasource.get();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        component.cancelEditedRows();
        LoadingService.hide();
      });
    }
  }

  public messageDeleteThermometerOpportunity({ event }: any) {
    Utilities.showConfirmDeleteMessage(event, 'CrmTermometroDeOportunidadeController.deleteThermometerOpportunity');
  }

  public deleteThermometerOpportunity() {
    const component = Metadata.getInstance<TekGrid>('tek_grid_crm_termometro_de_oportunidade_grid');
    LoadingService.show();
    Http.post('/deleteThermometerOpportunity', {
      NRTO: component.selectedRows.map((element: any) => element.NRTERMOPORTUNI),
    }).then(() => {
      Utilities.showSuccessMessage();
      component.datasource.get();
    }).catch((error: any) => {
      error.showErrorMessage();
    }).finally(() => {
      component.selectedRows = [];
      LoadingService.hide();
      DialogService.hide();
    });
  }
}
