import { Utilities } from '@/util/Utilities';
import {
  AlertService,
  DialogService,
  LoadingService,
  Modal, ModalService,
  Select, Tabs, TextInput, Textarea,
} from '@zeedhi/common';
import {
  Http, IEventParam,
  Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class CrmTipoRevendaController {
  public visibilidade: boolean = true;

  private NRTIPOREVENDA = '';

  private modalAdicionar!: Modal;

  public currentRow: any = {};

  get visibilidadeDoFrame() {
    return !this.visibilidade;
  }

  public alertMessage(message: string, type: string) {
    AlertService.show({
      name: 'basic-alert',
      text: message,
      top: true,
      left: false,
      timeout: 3000,
      color: type,
    });
  }

  public visualizarDadosGerais({ row }: any) {
    this.visibilidade = false;
    this.currentRow = row;
    this.recarregarTabs();
  }

  public recarregarTabs(): void {
    const setOfTabs = Metadata.getInstance<Tabs>('tabs-resale-type');
    if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-general-data') {
      this.montarInformacoesDadosGerais();
    } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-data-commission') {
      setTimeout(() => {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-data-commission');
        this.setFiltroGrid({ component: grid });
      }, 1000);
    }
  }

  public montarInformacoesDadosGerais() {
    Metadata.getInstance<TextInput>('NRTIPOREVENDA').value = this.currentRow.NRTIPOREVENDA;
    Metadata.getInstance<TextInput>('IDATIVO').value = this.currentRow.IDATIVO;
    Metadata.getInstance<Textarea>('DSTIPOREVENDA').value = this.currentRow.DSTIPOREVENDA;
    Metadata.getInstance<Select>('DSPRECBASESERV').value = this.currentRow.CDPRECBASESERV;
    Metadata.getInstance<Select>('DSMODPGTO').value = this.currentRow.CDMODPGTO;
  }

  public setFiltroGrid({ component }: any) {
    const nrcomissaotprevenda = this.currentRow.NRCOMISSAOTPREVENDA;
    component.datasource.setFilter({
      NRCOMISSAOTPREVENDA: nrcomissaotprevenda,
    });
  }

  public voltarPaginaInicial() {
    this.visibilidade = true;
  }

  public async openModalNewResaleType() {
    this.modalAdicionar = await ModalService.createFromJson('TipoDeRevenda/Modal/crm00040_000_modal_new_resale_type', true);
    await this.modalAdicionar.show();
  }

  public fecharModal() {
    this.modalAdicionar.hide();
    this.modalAdicionar?.destroy();
  }

  public saveNewResaleType({ event }: IEventParam<TekGrid>) {
    LoadingService.show();
    event!.preventDefault();
    const MDC = {
      NRTIPOREVENDA: this.NRTIPOREVENDA,
      IDATIVO: Metadata.getInstance<Select>('IDATIVO-MODAL').value,
      CDMODPGTO: Metadata.getInstance<Select>('DSMODPGTO-MODAL').value,
      CDPRECBASESERV: Metadata.getInstance<Select>('DSPRECBASESERV-MODAL').value,
      DSTIPOREVENDA: Metadata.getInstance<TextInput>('DSTIPOREVENDA-MODAL').value,
    };
    Http.post('/saveNewResaleType', MDC).then(() => {
      Utilities.showSuccessMessage();
      this.fecharModal();
      this.recarregarTabs();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public saveEditResaleTypeModal() {
    LoadingService.show();
    const MDC = {
      NRTIPOREVENDA: Metadata.getInstance<TextInput>('NRTIPOREVENDA').value,
      IDATIVO: Metadata.getInstance<Select>('IDATIVO').value,
      CDMODPGTO: Metadata.getInstance<Select>('DSMODPGTO').value,
      CDPRECBASESERV: Metadata.getInstance<Select>('DSPRECBASESERV').value,
      DSTIPOREVENDA: Metadata.getInstance<TextInput>('DSTIPOREVENDA').value,
    };
    Http.post('/updateResaleType', MDC).then(() => {
      this.recarregarTabs();
      Utilities.showSuccessMessage();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public confirmDeletionResaleType() {
    DialogService.show({
      name: 'dialog-agrupamento',
      type: 'warning',
      title: 'Atenção',
      text: 'Deseja realizar a exclusão desse tipo de revenda?.<br>',
      buttons: [
        {
          name: 'NAO',
          component: 'ZdButton',
          outline: true,
          label: 'Não',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'SIM',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.deteResaleTypeModal();
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  public deteResaleTypeModal() {
    LoadingService.show();
    const MDC = {
      NRTIPOREVENDA: Metadata.getInstance<TextInput>('NRTIPOREVENDA').value,
    };
    Http.post('/deleteResaleType', MDC).then(() => {
      this.recarregarTabs();
      Utilities.showSuccessMessage();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public addNewRow({ component }: IEventParam<TekGrid>) {
    const newId = 0;
    component.addNewRow({ NRSEQTPATIVCLI: newId, IDATIVO: 'S' }, 'start');
  }
}
